@@ -1,15 +1,10 @@
 /*
  * SNMP supplies functions for CUPS.
  *
- * Copyright 2008-2015 by Apple Inc.
+ * Copyright © 2008-2015 by Apple Inc.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * "LICENSE" which should have been included with this file.  If this
- * file is missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -445,6 +440,7 @@ backend_init_supplies(
 {
   int		i,			/* Looping var */
 		type;			/* Current marker type */
+  const char	*community;		/* SNMP community name */
   cups_file_t	*cachefile;		/* Cache file */
   const char	*cachedir;		/* CUPS_CACHEDIR value */
   char		addrstr[1024],		/* Address string */
@@ -512,6 +508,10 @@ backend_init_supplies(
   * See if we should be getting supply levels via SNMP...
   */
 
+  community = _cupsSNMPDefaultCommunity();
+  if (!*community)
+    return;
+
   if ((ppd = ppdOpenFile(getenv("PPD"))) == NULL ||
       ((ppdattr = ppdFindAttr(ppd, "cupsSNMPSupplies", NULL)) != NULL &&
        ppdattr->value && _cups_strcasecmp(ppdattr->value, "true")))
@@ -533,7 +533,7 @@ backend_init_supplies(
   */
 
   if (!_cupsSNMPWrite(snmp_fd, addr, CUPS_SNMP_VERSION_1,
-		     _cupsSNMPDefaultCommunity(), CUPS_ASN1_GET_REQUEST, 1,
+		     community, CUPS_ASN1_GET_REQUEST, 1,
 		     hrDeviceDescr))
     return;
 
@@ -610,7 +610,7 @@ backend_init_supplies(
 
 
     if (!_cupsSNMPWrite(snmp_fd, &current_addr, CUPS_SNMP_VERSION_1,
-			_cupsSNMPDefaultCommunity(), CUPS_ASN1_GET_REQUEST, 1,
+			community, CUPS_ASN1_GET_REQUEST, 1,
 			prtGeneralCurrentLocalization))
       return;
 
@@ -631,7 +631,7 @@ backend_init_supplies(
 
 
     if (!_cupsSNMPWrite(snmp_fd, &current_addr, CUPS_SNMP_VERSION_1,
-			_cupsSNMPDefaultCommunity(), CUPS_ASN1_GET_REQUEST, 1,
+			community, CUPS_ASN1_GET_REQUEST, 1,
 			oid))
       return;
 
@@ -656,7 +656,7 @@ backend_init_supplies(
     */
 
     _cupsSNMPWalk(snmp_fd, &current_addr, CUPS_SNMP_VERSION_1,
-		  _cupsSNMPDefaultCommunity(), prtMarkerSuppliesEntry,
+		  community, prtMarkerSuppliesEntry,
 		  CUPS_SUPPLY_TIMEOUT, backend_walk_cb, NULL);
   }
 
@@ -690,7 +690,7 @@ backend_init_supplies(
     strlcpy(supplies[i].color, "none", sizeof(supplies[i].color));
 
   _cupsSNMPWalk(snmp_fd, &current_addr, CUPS_SNMP_VERSION_1,
-                _cupsSNMPDefaultCommunity(), prtMarkerColorantValue,
+                community, prtMarkerColorantValue,
 	        CUPS_SUPPLY_TIMEOUT, backend_walk_cb, NULL);
 
  /*
