@@ -1,6 +1,7 @@
 /*
  * PPD file routines for CUPS.
  *
+ * Copyright © 2021-2022 by OpenPrinting.
  * Copyright © 2007-2019 by Apple Inc.
  * Copyright © 1997-2007 by Easy Software Products, all rights reserved.
  *
@@ -125,6 +126,7 @@ ppdClose(ppd_file_t *ppd)		/* I - PPD file record */
   free(ppd->lang_encoding);
   free(ppd->nickname);
   free(ppd->patches);
+  free(ppd->emulations);
   free(ppd->jcl_begin);
   free(ppd->jcl_end);
   free(ppd->jcl_ps);
@@ -871,15 +873,15 @@ _ppdOpen(
       ppd_decode(ppd->jcl_ps);		/* Decode quoted string */
     }
     else if (!strcmp(keyword, "AccurateScreensSupport"))
-      ppd->accurate_screens = !strcmp(string, "True");
+      ppd->accurate_screens = !strcasecmp(string, "True");
     else if (!strcmp(keyword, "ColorDevice"))
-      ppd->color_device = !strcmp(string, "True");
+      ppd->color_device = !strcasecmp(string, "True");
     else if (!strcmp(keyword, "ContoneOnly"))
-      ppd->contone_only = !strcmp(string, "True");
+      ppd->contone_only = !strcasecmp(string, "True");
     else if (!strcmp(keyword, "cupsFlipDuplex"))
-      ppd->flip_duplex = !strcmp(string, "True");
+      ppd->flip_duplex = !strcasecmp(string, "True");
     else if (!strcmp(keyword, "cupsManualCopies"))
-      ppd->manual_copies = !strcmp(string, "True");
+      ppd->manual_copies = !strcasecmp(string, "True");
     else if (!strcmp(keyword, "cupsModelNumber"))
       ppd->model_number = atoi(string);
     else if (!strcmp(keyword, "cupsColorProfile"))
@@ -1496,6 +1498,27 @@ _ppdOpen(
 	goto error;
       }
 
+      if (option && (!_cups_strcasecmp(option->defchoice, "custom") || !_cups_strncasecmp(option->defchoice, "custom.", 7)))
+      {
+       /*
+	* "*DefaultOption: Custom..." may set the default to a custom value
+	* or (for a very small number of incompatible PPD files) select a
+	* standard choice for the option, which CUPS renames to "_Custom..."
+	* to avoid compatibility issues.  See which this is...
+	*/
+
+        char tchoice[PPD_MAX_NAME];	/* Temporary choice name */
+
+	snprintf(tchoice, sizeof(tchoice), "_%s", option->defchoice);
+
+	if (ppdFindChoice(option, tchoice))
+	{
+	  strlcpy(option->defchoice, tchoice, sizeof(option->defchoice));
+
+	  DEBUG_printf(("2_ppdOpen: Reset Default%s to %s...", option->keyword, tchoice));
+	}
+      }
+
       option = NULL;
 
       free(string);
@@ -1510,6 +1533,27 @@ _ppdOpen(
 	goto error;
       }
 
+      if (option && (!_cups_strcasecmp(option->defchoice, "custom") || !_cups_strncasecmp(option->defchoice, "custom.", 7)))
+      {
+       /*
+	* "*DefaultOption: Custom..." may set the default to a custom value
+	* or (for a very small number of incompatible PPD files) select a
+	* standard choice for the option, which CUPS renames to "_Custom..."
+	* to avoid compatibility issues.  See which this is...
+	*/
+
+        char tchoice[PPD_MAX_NAME];	/* Temporary choice name */
+
+	snprintf(tchoice, sizeof(tchoice), "_%s", option->defchoice);
+
+	if (ppdFindChoice(option, tchoice))
+	{
+	  strlcpy(option->defchoice, tchoice, sizeof(option->defchoice));
+
+	  DEBUG_printf(("2_ppdOpen: Reset Default%s to %s...", option->keyword, tchoice));
+	}
+      }
+
       option = NULL;
 
       free(string);
@@ -1668,11 +1712,9 @@ _ppdOpen(
         * Set the default as part of the current option...
 	*/
 
-        DEBUG_printf(("2_ppdOpen: Setting %s to %s...", keyword, string));
+	strlcpy(option->defchoice, string, sizeof(option->defchoice));
 
-        strlcpy(option->defchoice, string, sizeof(option->defchoice));
-
-        DEBUG_printf(("2_ppdOpen: %s is now %s...", keyword, option->defchoice));
+        DEBUG_printf(("2_ppdOpen: Set %s to %s...", keyword, option->defchoice));
       }
       else
       {
@@ -1682,11 +1724,27 @@ _ppdOpen(
 
         ppd_option_t	*toption;	/* Temporary option */
 
-
         if ((toption = ppdFindOption(ppd, keyword + 7)) != NULL)
 	{
-	  DEBUG_printf(("2_ppdOpen: Setting %s to %s...", keyword, string));
-	  strlcpy(toption->defchoice, string, sizeof(toption->defchoice));
+	  if (!_cups_strcasecmp(string, "custom") || !_cups_strncasecmp(string, "custom.", 7))
+	  {
+	   /*
+	    * "*DefaultOption: Custom..." may set the default to a custom value
+	    * or (for a very small number of incompatible PPD files) select a
+	    * standard choice for the option, which CUPS renames to "_Custom..."
+	    * to avoid compatibility issues.  See which this is...
+	    */
+
+	    snprintf(toption->defchoice, sizeof(toption->defchoice), "_%s", string);
+	    if (!ppdFindChoice(toption, toption->defchoice))
+	      strlcpy(toption->defchoice, string, sizeof(toption->defchoice));
+	  }
+	  else
+	  {
+	    strlcpy(toption->defchoice, string, sizeof(toption->defchoice));
+	  }
+
+	  DEBUG_printf(("2_ppdOpen: Set %s to %s...", keyword, toption->defchoice));
 	}
       }
     }
@@ -1719,8 +1777,7 @@ _ppdOpen(
                      constraint->choice1, constraint->option2,
 		     constraint->choice2))
       {
-        case 0 : /* Error */
-	case 1 : /* Error */
+        default : /* Error */
 	    pg->ppd_status = PPD_BAD_UI_CONSTRAINTS;
 	    goto error;
 
@@ -2336,8 +2393,16 @@ ppd_add_attr(ppd_file_t *ppd,		/* I - PPD file data */
   * Copy data over...
   */
 
+  if (!_cups_strcasecmp(spec, "custom") || !_cups_strncasecmp(spec, "custom.", 7))
+  {
+    temp->spec[0] = '_';
+    strlcpy(temp->spec + 1, spec, sizeof(temp->spec) - 1);
+  }
+  else {
+      strlcpy(temp->spec, spec, sizeof(temp->spec));
+  }
+
   strlcpy(temp->name, name, sizeof(temp->name));
-  strlcpy(temp->spec, spec, sizeof(temp->spec));
   strlcpy(temp->text, text, sizeof(temp->text));
   temp->value = (char *)value;
 
@@ -2840,7 +2905,7 @@ ppd_hash_option(ppd_option_t *option)	/* I - Option */
 
 
   for (hash = option->keyword[0], k = option->keyword + 1; *k;)
-    hash = 33 * hash + *k++;
+    hash = (int)(33U * (unsigned)hash) + *k++;
 
   return (hash & 511);
 }
@@ -3365,12 +3430,12 @@ ppd_update_filters(ppd_file_t     *ppd,	/* I - PPD file */
 		srctype[256],
 		dstsuper[16],		/* Destination MIME media type */
 		dsttype[256],
-		program[1024],		/* Command to run */
 		*ptr,			/* Pointer into command to run */
 		buffer[1024],		/* Re-written cupsFilter value */
 		**filter;		/* Current filter */
   int		cost;			/* Cost of filter */
 
+  char		program[1024] = { 0 };	/* Command to run */
 
   DEBUG_printf(("4ppd_update_filters(ppd=%p, cg=%p)", ppd, pg));
 
