@@ -1,6 +1,7 @@
 /*
  * Simulated client test program for CUPS.
  *
+ * Copyright © 2020-2022 by OpenPrinting
  * Copyright © 2017-2019 by Apple Inc.
  *
  * Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -24,7 +25,7 @@
  * Constants...
  */
 
-#define MAX_CLIENTS	16		/* Maximum number of client threads */
+#define MAX_CLIENTS	100		/* Maximum number of client threads */
 
 
 /*
@@ -44,12 +45,12 @@ typedef struct _client_data_s
   int			grayscale,	/* Force grayscale? */
 			keepfile;	/* Keep temporary file? */
   ipp_pstate_t		printer_state;	/* Current printer state */
-  char                  printer_state_reasons[1024];
-                                        /* Current printer-state-reasons */
+  char			printer_state_reasons[1024];
+					/* Current printer-state-reasons */
   int			job_id; 	/* Job ID for submitted job */
   ipp_jstate_t		job_state;	/* Current job state */
-  char                  job_state_reasons[1024];
-                                        /* Current job-state-reasons */
+  char			job_state_reasons[1024];
+					/* Current job-state-reasons */
 } _client_data_t;
 
 
@@ -66,12 +67,12 @@ static int		verbosity = 0;
  * Local functions...
  */
 
-static const char       *make_raster_file(ipp_t *response, int grayscale, char *tempname, size_t tempsize, const char **format);
-static void	        *monitor_printer(_client_data_t *data);
+static const char	*make_raster_file(ipp_t *response, int grayscale, char *tempname, size_t tempsize, const char **format);
+static void		*monitor_printer(_client_data_t *data);
 static void		*run_client(_client_data_t *data);
-static void             show_attributes(const char *title, int request, ipp_t *ipp);
-static void             show_capabilities(ipp_t *response);
-static void             usage(void);
+static void		show_attributes(const char *title, int request, ipp_t *ipp);
+static void		show_capabilities(ipp_t *response);
+static void		usage(void);
 
 
 /*
@@ -84,9 +85,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 {
   int			i;		/* Looping var */
   const char		*opt;		/* Current option */
-  int			num_clients = 0,/* Number of clients to simulate */
-			clients_started = 0;
-					/* Number of clients that have been started */
+  int			num_clients = 0;/* Number of clients to simulate */
   char			scheme[32],     /* URI scheme */
 			userpass[256],  /* Username:password */
 			hostname[256],  /* Hostname */
@@ -241,7 +240,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   data.hostname = hostname;
   data.resource = resource;
 
-  while (clients_started < num_clients)
+  while (client_count < num_clients)
   {
     _cupsMutexLock(&client_mutex);
     if (client_count < MAX_CLIENTS)
@@ -396,15 +395,7 @@ make_raster_file(ipp_t      *response,  /* I - Printer attributes */
   * Figure out the the media, resolution, and color mode...
   */
 
-  if ((attr = ippFindAttribute(response, "media-default", IPP_TAG_KEYWORD)) != NULL)
-  {
-   /*
-    * Use default media...
-    */
-
-    media = pwgMediaForPWG(ippGetString(attr, 0, NULL));
-  }
-  else if ((attr = ippFindAttribute(response, "media-ready", IPP_TAG_KEYWORD)) != NULL)
+  if ((attr = ippFindAttribute(response, "media-ready", IPP_TAG_KEYWORD)) != NULL)
   {
    /*
     * Use ready media...
@@ -417,6 +408,14 @@ make_raster_file(ipp_t      *response,  /* I - Printer attributes */
     else
       media = pwgMediaForPWG(ippGetString(attr, 0, NULL));
   }
+  else if ((attr = ippFindAttribute(response, "media-default", IPP_TAG_KEYWORD)) != NULL)
+  {
+   /*
+    * Use default media...
+    */
+
+    media = pwgMediaForPWG(ippGetString(attr, 0, NULL));
+  }
   else
   {
     puts("No default or ready media reported by printer, aborting.");
@@ -486,15 +485,15 @@ make_raster_file(ipp_t      *response,  /* I - Printer attributes */
 
   header.cupsInteger[CUPS_RASTER_PWG_TotalPageCount] = 1;
 
-  if (header.cupsWidth > (4 * header.HWResolution[0]))
+  if (header.cupsWidth > (2 * header.HWResolution[0]))
   {
     xoff = header.HWResolution[0] / 2;
     yoff = header.HWResolution[1] / 2;
   }
   else
   {
-    xoff = 0;
-    yoff = 0;
+    xoff = header.HWResolution[0] / 4;
+    yoff = header.HWResolution[1] / 4;
   }
 
   xrep = (header.cupsWidth - 2 * xoff) / 140;
@@ -600,9 +599,11 @@ make_raster_file(ipp_t      *response,  /* I - Printer attributes */
 
   memset(line, 0xff, header.cupsBytesPerLine);
 
-  for (y = 0; y < header.cupsHeight; y ++)
+  for (; y < header.cupsHeight; y ++)
     cupsRasterWritePixels(ras, line, header.cupsBytesPerLine);
 
+  free(line);
+
   cupsRasterClose(ras);
 
   close(fd);
@@ -768,11 +769,8 @@ run_client(
   ipp_attribute_t *attr;		/* Attribute in response */
   static const char * const pattrs[] =  /* Printer attributes we are interested in */
   {
-    "job-template",
-    "printer-defaults",
-    "printer-description",
-    "media-col-database",
-    "media-col-ready"
+    "all",
+    "media-col-database"
   };
 
 
@@ -859,33 +857,47 @@ run_client(
   * Create a job and wait for completion...
   */
 
-  request = ippNewRequest(IPP_OP_CREATE_JOB);
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, ldata.uri);
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name", NULL, cupsUser());
+  do
+  {
+    request = ippNewRequest(IPP_OP_CREATE_JOB);
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, ldata.uri);
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "requesting-user-name", NULL, cupsUser());
 
-  if ((name = strrchr(ldata.docfile, '/')) != NULL)
-    name ++;
-  else
-    name = ldata.docfile;
+    if ((name = strrchr(ldata.docfile, '/')) != NULL)
+      name ++;
+    else
+      name = ldata.docfile;
 
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL, name);
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME, "job-name", NULL, name);
 
-  if (verbosity)
-    show_attributes("Create-Job request", 1, request);
+    if (verbosity)
+      show_attributes("Create-Job request", 1, request);
 
-  response = cupsDoRequest(http, request, ldata.resource);
+    response = cupsDoRequest(http, request, ldata.resource);
 
-  if (verbosity)
-    show_attributes("Create-Job response", 0, response);
+    if (verbosity)
+      show_attributes("Create-Job response", 0, response);
+
+    if (cupsLastError() == IPP_STATUS_ERROR_BUSY)
+    {
+      puts("Printer is busy - retrying in 5 seconds...");
+      sleep(5);
+      ippDelete(response);
+      response = NULL;
+    }
+    else if (cupsLastError() >= IPP_STATUS_REDIRECTION_OTHER_SITE)
+    {
+      printf("Unable to create print job: %s\n", cupsLastErrorString());
+
+      ldata.job_state = IPP_JSTATE_ABORTED;
+      ippDelete(response);
+      response = NULL;
+    }
+  }
+  while (cupsLastError() == IPP_STATUS_ERROR_BUSY);
 
   if (cupsLastError() >= IPP_STATUS_REDIRECTION_OTHER_SITE)
-  {
-    printf("Unable to create print job: %s\n", cupsLastErrorString());
-
-    ldata.job_state = IPP_JSTATE_ABORTED;
-
     goto cleanup;
-  }
 
   if ((attr = ippFindAttribute(response, "job-id", IPP_TAG_INTEGER)) == NULL)
   {
