@@ -1,16 +1,12 @@
 /*
  * User, system, and password routines for CUPS.
  *
- * Copyright 2007-2017 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products.
+ * Copyright © 2021-2022 by OpenPrinting.
+ * Copyright © 2007-2019 by Apple Inc.
+ * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
- *
- * This file is subject to the Apple OS-Developed Software exception.
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -18,6 +14,7 @@
  */
 
 #include "cups-private.h"
+#include "debug-internal.h"
 #include <stdlib.h>
 #include <sys/stat.h>
 #ifdef _WIN32
@@ -27,6 +24,9 @@
 #  include <termios.h>
 #  include <sys/utsname.h>
 #endif /* _WIN32 */
+#ifdef __APPLE__
+#  include <sys/sysctl.h>
+#endif /* __APPLE__ */
 
 
 /*
@@ -34,14 +34,27 @@
  */
 
 #ifdef __APPLE__
-#  define kCUPSPrintingPrefs	CFSTR("org.cups.PrintingPrefs")
-#  define kAllowAnyRootKey	CFSTR("AllowAnyRoot")
-#  define kAllowExpiredCertsKey	CFSTR("AllowExpiredCerts")
-#  define kEncryptionKey	CFSTR("Encryption")
-#  define kGSSServiceNameKey	CFSTR("GSSServiceName")
-#  define kSSLOptionsKey	CFSTR("SSLOptions")
-#  define kTrustOnFirstUseKey	CFSTR("TrustOnFirstUse")
-#  define kValidateCertsKey	CFSTR("ValidateCerts")
+#  if TARGET_OS_OSX
+#    define kCUPSPrintingPrefs	CFSTR("org.cups.PrintingPrefs")
+#    define kPREFIX		""
+#  else
+#    define kCUPSPrintingPrefs	CFSTR(".GlobalPreferences")
+#    define kPREFIX		"AirPrint"
+#  endif /* TARGET_OS_OSX */
+#  define kDigestOptionsKey	CFSTR(kPREFIX "DigestOptions")
+#  define kUserKey		CFSTR(kPREFIX "User")
+#  define kUserAgentTokensKey	CFSTR(kPREFIX "UserAgentTokens")
+#  define kAllowAnyRootKey	CFSTR(kPREFIX "AllowAnyRoot")
+#  define kAllowExpiredCertsKey	CFSTR(kPREFIX "AllowExpiredCerts")
+#  define kEncryptionKey	CFSTR(kPREFIX "Encryption")
+#  define kGSSServiceNameKey	CFSTR(kPREFIX "GSSServiceName")
+#  define kSSLOptionsKey	CFSTR(kPREFIX "SSLOptions")
+#  define kTrustOnFirstUseKey	CFSTR(kPREFIX "TrustOnFirstUse")
+#  define kValidateCertsKey	CFSTR(kPREFIX "ValidateCerts")
+/* Deprecated */
+#  define kAllowRC4		CFSTR(kPREFIX "AllowRC4")
+#  define kAllowSSL3		CFSTR(kPREFIX "AllowSSL3")
+#  define kAllowDH		CFSTR(kPREFIX "AllowDH")
 #endif /* __APPLE__ */
 
 #define _CUPS_PASSCHAR	'*'		/* Character that is echoed for password */
@@ -53,11 +66,13 @@
 
 typedef struct _cups_client_conf_s	/**** client.conf config data ****/
 {
-#ifdef HAVE_SSL
+  _cups_digestoptions_t	digestoptions;	/* DigestOptions values */
+  _cups_uatokens_t	uatokens;	/* UserAgentTokens values */
+#ifdef HAVE_TLS
   int			ssl_options,	/* SSLOptions values */
 			ssl_min_version,/* Minimum SSL/TLS version */
 			ssl_max_version;/* Maximum SSL/TLS version */
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
   int			trust_first,	/* Trust on first use? */
 			any_root,	/* Allow any (e.g., self-signed) root */
 			expired_certs,	/* Allow expired certs */
@@ -78,7 +93,9 @@ typedef struct _cups_client_conf_s	/**** client.conf config data ****/
  */
 
 #ifdef __APPLE__
+#  ifdef HAVE_TLS
 static int	cups_apple_get_boolean(CFStringRef key, int *value);
+#  endif /* HAVE_TLS */
 static int	cups_apple_get_string(CFStringRef key, char *value, size_t valsize);
 #endif /* __APPLE__ */
 static int	cups_boolean_value(const char *value);
@@ -86,14 +103,16 @@ static void	cups_finalize_client_conf(_cups_client_conf_t *cc);
 static void	cups_init_client_conf(_cups_client_conf_t *cc);
 static void	cups_read_client_conf(cups_file_t *fp, _cups_client_conf_t *cc);
 static void	cups_set_default_ipp_port(_cups_globals_t *cg);
+static void	cups_set_digestoptions(_cups_client_conf_t *cc, const char *value);
 static void	cups_set_encryption(_cups_client_conf_t *cc, const char *value);
 #ifdef HAVE_GSSAPI
 static void	cups_set_gss_service_name(_cups_client_conf_t *cc, const char *value);
 #endif /* HAVE_GSSAPI */
 static void	cups_set_server_name(_cups_client_conf_t *cc, const char *value);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 static void	cups_set_ssl_options(_cups_client_conf_t *cc, const char *value);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
+static void	cups_set_uatokens(_cups_client_conf_t *cc, const char *value);
 static void	cups_set_user(_cups_client_conf_t *cc, const char *value);
 
 
@@ -255,10 +274,10 @@ cupsSetCredentials(
   if (cupsArrayCount(credentials) < 1)
     return (-1);
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
   _httpFreeCredentials(cg->tls_credentials);
   cg->tls_credentials = _httpCreateCredentials(credentials);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
   return (cg->tls_credentials ? 0 : -1);
 }
@@ -290,6 +309,43 @@ cupsSetEncryption(http_encryption_t e)	/* I - New encryption preference */
 }
 
 
+/*
+ * 'cupsSetOAuthCB()' - Set the OAuth 2.0 callback for CUPS.
+ *
+ * This function sets the OAuth 2.0 callback for the various CUPS APIs that
+ * send HTTP requests. Pass @code NULL@ to restore the default (console-based)
+ * callback.
+ *
+ * The OAuth callback receives the HTTP connection, realm name, scope name (if
+ * any), resource path, and the "user_data" pointer for each request that
+ * requires an OAuth access token. The function then returns either the Bearer
+ * token string or `NULL` if no authorization could be obtained.
+ *
+ * Beyond reusing the Bearer token for subsequent requests on the same HTTP
+ * connection, no caching of the token is done by the CUPS library.  The
+ * callback can determine whether to refresh a cached token by examining any
+ * existing token returned by the @link httpGetAuthString@ function.
+ *
+ * Note: The current OAuth callback is tracked separately for each thread in a
+ * program. Multi-threaded programs that override the callback need to do so in
+ * each thread for the same callback to be used.
+ *
+ * @since CUPS 2.4@
+ */
+
+void
+cupsSetOAuthCB(
+    cups_oauth_cb_t cb,			/* I - Callback function */
+    void            *user_data)		/* I - User data pointer */
+{
+  _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
+
+
+  cg->oauth_cb   = cb;
+  cg->oauth_data = user_data;
+}
+
+
 /*
  * 'cupsSetPasswordCB()' - Set the password callback for CUPS.
  *
@@ -492,7 +548,12 @@ cupsSetUserAgent(const char *user_agent)/* I - User-Agent string or @code NULL@
 					/* Thread globals */
 #ifdef _WIN32
   SYSTEM_INFO		sysinfo;	/* System information */
-  OSVERSIONINFO		version;	/* OS version info */
+  OSVERSIONINFOA	version;	/* OS version info */
+  const char		*machine;	/* Hardware/machine name */
+#elif defined(__APPLE__)
+  struct utsname	name;		/* uname info */
+  char			version[256];	/* macOS/iOS version */
+  size_t		len;		/* Length of value */
 #else
   struct utsname	name;		/* uname info */
 #endif /* _WIN32 */
@@ -504,30 +565,103 @@ cupsSetUserAgent(const char *user_agent)/* I - User-Agent string or @code NULL@
     return;
   }
 
+  if (cg->uatokens < _CUPS_UATOKENS_OS)
+  {
+    switch (cg->uatokens)
+    {
+      default :
+      case _CUPS_UATOKENS_NONE :
+	  cg->user_agent[0] = '\0';
+	  break;
+      case _CUPS_UATOKENS_PRODUCT_ONLY :
+	  strlcpy(cg->user_agent, "CUPS IPP", sizeof(cg->user_agent));
+	  break;
+      case _CUPS_UATOKENS_MAJOR :
+	  snprintf(cg->user_agent, sizeof(cg->user_agent), "CUPS/%d IPP/2", CUPS_VERSION_MAJOR);
+	  break;
+      case _CUPS_UATOKENS_MINOR :
+	  snprintf(cg->user_agent, sizeof(cg->user_agent), "CUPS/%d.%d IPP/2.1", CUPS_VERSION_MAJOR, CUPS_VERSION_MINOR);
+	  break;
+      case _CUPS_UATOKENS_MINIMAL :
+	  strlcpy(cg->user_agent, CUPS_MINIMAL " IPP/2.1", sizeof(cg->user_agent));
+	  break;
+    }
+  }
+
 #ifdef _WIN32
+ /*
+  * Gather Windows version information for the User-Agent string...
+  */
+
   version.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-  GetVersionEx(&version);
+  GetVersionExA(&version);
   GetNativeSystemInfo(&sysinfo);
 
-  snprintf(cg->user_agent, sizeof(cg->user_agent),
-           CUPS_MINIMAL " (Windows %d.%d; %s) IPP/2.0",
-	   version.dwMajorVersion, version.dwMinorVersion,
-	   sysinfo.wProcessorArchitecture
-	       == PROCESSOR_ARCHITECTURE_AMD64 ? "amd64" :
-	       sysinfo.wProcessorArchitecture
-		   == PROCESSOR_ARCHITECTURE_ARM ? "arm" :
-	       sysinfo.wProcessorArchitecture
-		   == PROCESSOR_ARCHITECTURE_IA64 ? "ia64" :
-	       sysinfo.wProcessorArchitecture
-		   == PROCESSOR_ARCHITECTURE_INTEL ? "intel" :
-	       "unknown");
+  switch (sysinfo.wProcessorArchitecture)
+  {
+    case PROCESSOR_ARCHITECTURE_AMD64 :
+        machine = "amd64";
+        break;
+
+    case PROCESSOR_ARCHITECTURE_ARM :
+        machine = "arm";
+        break;
+
+    case PROCESSOR_ARCHITECTURE_IA64 :
+        machine = "ia64";
+        break;
+
+    case PROCESSOR_ARCHITECTURE_INTEL :
+        machine = "intel";
+        break;
+
+    default :
+        machine = "unknown";
+        break;
+  }
+
+  if (cg->uatokens == _CUPS_UATOKENS_OS)
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (Windows %d.%d) IPP/2.0", version.dwMajorVersion, version.dwMinorVersion);
+  else
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (Windows %d.%d; %s) IPP/2.0", version.dwMajorVersion, version.dwMinorVersion, machine);
+
+#elif defined(__APPLE__)
+ /*
+  * Gather macOS/iOS version information for the User-Agent string...
+  */
 
-#else
   uname(&name);
 
-  snprintf(cg->user_agent, sizeof(cg->user_agent),
-           CUPS_MINIMAL " (%s %s; %s) IPP/2.0",
-	   name.sysname, name.release, name.machine);
+  len = sizeof(version) - 1;
+  if (!sysctlbyname("kern.osproductversion", version, &len, NULL, 0))
+    version[len] = '\0';
+  else
+    strlcpy(version, "unknown", sizeof(version));
+
+#  if TARGET_OS_OSX
+  if (cg->uatokens == _CUPS_UATOKENS_OS)
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (macOS %s) IPP/2.0", version);
+  else
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (macOS %s; %s) IPP/2.0", version, name.machine);
+
+#  else
+  if (cg->uatokens == _CUPS_UATOKENS_OS)
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (iOS %s) IPP/2.0", version);
+  else
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (iOS %s; %s) IPP/2.0", version, name.machine);
+#  endif /* TARGET_OS_OSX */
+
+#else
+ /*
+  * Gather generic UNIX version information for the User-Agent string...
+  */
+
+  uname(&name);
+
+  if (cg->uatokens == _CUPS_UATOKENS_OS)
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (%s %s) IPP/2.0", name.sysname, name.release);
+  else
+    snprintf(cg->user_agent, sizeof(cg->user_agent), CUPS_MINIMAL " (%s %s; %s) IPP/2.0", name.sysname, name.release, name.machine);
 #endif /* _WIN32 */
 }
 
@@ -877,7 +1011,6 @@ void
 _cupsSetDefaults(void)
 {
   cups_file_t	*fp;			/* File */
-  const char	*home;			/* Home directory of user */
   char		filename[1024];		/* Filename */
   _cups_client_conf_t cc;		/* client.conf values */
   _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
@@ -903,19 +1036,18 @@ _cupsSetDefaults(void)
     cupsFileClose(fp);
   }
 
-#  ifdef HAVE_GETEUID
-  if ((geteuid() == getuid() || !getuid()) && getegid() == getgid() && (home = getenv("HOME")) != NULL)
-#  elif !defined(_WIN32)
-  if (getuid() && (home = getenv("HOME")) != NULL)
-#  else
-  if ((home = getenv("HOME")) != NULL)
-#  endif /* HAVE_GETEUID */
+  if (cg->home)
   {
    /*
     * Look for ~/.cups/client.conf...
     */
 
-    snprintf(filename, sizeof(filename), "%s/.cups/client.conf", home);
+#if _WIN32
+    snprintf(filename, sizeof(filename), "%s/AppData/Local/cups/client.conf", cg->home);
+#else
+    snprintf(filename, sizeof(filename), "%s/.cups/client.conf", cg->home);
+#endif // _WIN32
+
     if ((fp = cupsFileOpen(filename, "r")) != NULL)
     {
       cups_read_client_conf(fp, &cc);
@@ -929,6 +1061,8 @@ _cupsSetDefaults(void)
 
   cups_finalize_client_conf(&cc);
 
+  cg->uatokens = cc.uatokens;
+
   if (cg->encryption == (http_encryption_t)-1)
     cg->encryption = cc.encryption;
 
@@ -958,13 +1092,14 @@ _cupsSetDefaults(void)
   if (cg->validate_certs < 0)
     cg->validate_certs = cc.validate_certs;
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
   _httpTLSSetOptions(cc.ssl_options | _HTTP_TLS_SET_DEFAULT, cc.ssl_min_version, cc.ssl_max_version);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 }
 
 
 #ifdef __APPLE__
+#  ifdef HAVE_TLS
 /*
  * 'cups_apple_get_boolean()' - Get a boolean setting from the CUPS preferences.
  */
@@ -985,6 +1120,7 @@ cups_apple_get_boolean(
 
   return ((int)bval_set);
 }
+#  endif /* HAVE_TLS */
 
 
 /*
@@ -1086,17 +1222,22 @@ cups_finalize_client_conf(
 
   if (!cc->server_name[0])
   {
-#ifdef CUPS_DEFAULT_DOMAINSOCKET
    /*
     * If we are compiled with domain socket support, only use the
     * domain socket if it exists and has the right permissions...
     */
 
+#if defined(__APPLE__) && !TARGET_OS_OSX
+    cups_set_server_name(cc, "/private/var/run/printd");
+
+#else
+#  ifdef CUPS_DEFAULT_DOMAINSOCKET
     if (!access(CUPS_DEFAULT_DOMAINSOCKET, R_OK))
       cups_set_server_name(cc, CUPS_DEFAULT_DOMAINSOCKET);
     else
-#endif /* CUPS_DEFAULT_DOMAINSOCKET */
-      cups_set_server_name(cc, "localhost");
+#  endif /* CUPS_DEFAULT_DOMAINSOCKET */
+    cups_set_server_name(cc, "localhost");
+#endif /* __APPLE__ && !TARGET_OS_OSX */
   }
 
   if (!cc->user[0])
@@ -1109,15 +1250,16 @@ cups_finalize_client_conf(
     DWORD	size;			/* Size of string */
 
     size = sizeof(cc->user);
-    if (!GetUserName(cc->user, &size))
+    if (!GetUserNameA(cc->user, &size))
 #else
    /*
     * Try the USER environment variable as the default username...
     */
 
-    const char *envuser = getenv("USER");
-					/* Default username */
-    struct passwd *pw = NULL;		/* Account information */
+    const char *envuser = getenv("USER");	/* Default username */
+    struct passwd pw;				/* Account information */
+    struct passwd *result = NULL;		/* Auxiliary pointer */
+    _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
 
     if (envuser)
     {
@@ -1126,16 +1268,16 @@ cups_finalize_client_conf(
       * override things...  This makes sure that printing after doing su
       * or sudo records the correct username.
       */
-
-      if ((pw = getpwnam(envuser)) != NULL && pw->pw_uid != getuid())
-	pw = NULL;
+      getpwnam_r(envuser, &pw, cg->pw_buf, PW_BUF_SIZE, &result);
+      if (result && pw.pw_uid != getuid())
+        result = NULL;
     }
 
-    if (!pw)
-      pw = getpwuid(getuid());
+    if (!result)
+      getpwuid_r(getuid(), &pw, cg->pw_buf, PW_BUF_SIZE, &result);
 
-    if (pw)
-      strlcpy(cc->user, pw->pw_name, sizeof(cc->user));
+    if (result)
+      strlcpy(cc->user, pw.pw_name, sizeof(cc->user));
     else
 #endif /* _WIN32 */
     {
@@ -1166,10 +1308,16 @@ cups_init_client_conf(
 
   memset(cc, 0, sizeof(_cups_client_conf_t));
 
-#ifdef HAVE_SSL
+  cc->uatokens = _CUPS_UATOKENS_MINIMAL;
+
+#if defined(__APPLE__) && !TARGET_OS_OSX
+  cups_set_user(cc, "mobile");
+#endif /* __APPLE__ && !TARGET_OS_OSX */
+
+#ifdef HAVE_TLS
   cc->ssl_min_version = _HTTP_TLS_1_0;
   cc->ssl_max_version = _HTTP_TLS_MAX;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
   cc->encryption      = (http_encryption_t)-1;
   cc->trust_first     = -1;
   cc->any_root        = -1;
@@ -1181,8 +1329,9 @@ cups_init_client_conf(
   * everything...)
   */
 
-#if defined(__APPLE__) && defined(HAVE_SSL)
+#if defined(__APPLE__)
   char	sval[1024];			/* String value */
+#  ifdef HAVE_TLS
   int	bval;				/* Boolean value */
 
   if (cups_apple_get_boolean(kAllowAnyRootKey, &bval))
@@ -1195,14 +1344,40 @@ cups_init_client_conf(
     cups_set_encryption(cc, sval);
 
   if (cups_apple_get_string(kSSLOptionsKey, sval, sizeof(sval)))
+  {
     cups_set_ssl_options(cc, sval);
+  }
+  else
+  {
+    sval[0] = '\0';
+
+    if (cups_apple_get_boolean(kAllowRC4, &bval) && bval)
+      strlcat(sval, " AllowRC4", sizeof(sval));
+    if (cups_apple_get_boolean(kAllowSSL3, &bval) && bval)
+      strlcat(sval, " AllowSSL3", sizeof(sval));
+    if (cups_apple_get_boolean(kAllowDH, &bval) && bval)
+      strlcat(sval, " AllowDH", sizeof(sval));
+
+    if (sval[0])
+      cups_set_ssl_options(cc, sval);
+  }
 
   if (cups_apple_get_boolean(kTrustOnFirstUseKey, &bval))
     cc->trust_first = bval;
 
   if (cups_apple_get_boolean(kValidateCertsKey, &bval))
     cc->validate_certs = bval;
-#endif /* __APPLE__ && HAVE_SSL */
+#  endif /* HAVE_TLS */
+
+  if (cups_apple_get_string(kDigestOptionsKey, sval, sizeof(sval)))
+    cups_set_digestoptions(cc, sval);
+
+  if (cups_apple_get_string(kUserKey, sval, sizeof(sval)))
+    strlcpy(cc->user, sval, sizeof(cc->user));
+
+  if (cups_apple_get_string(kUserAgentTokensKey, sval, sizeof(sval)))
+    cups_set_uatokens(cc, sval);
+#endif /* __APPLE__ */
 }
 
 
@@ -1227,7 +1402,9 @@ cups_read_client_conf(
   linenum = 0;
   while (cupsFileGetConf(fp, line, sizeof(line), &value, &linenum))
   {
-    if (!_cups_strcasecmp(line, "Encryption") && value)
+    if (!_cups_strcasecmp(line, "DigestOptions") && value)
+      cups_set_digestoptions(cc, value);
+    else if (!_cups_strcasecmp(line, "Encryption") && value)
       cups_set_encryption(cc, value);
 #ifndef __APPLE__
    /*
@@ -1239,6 +1416,8 @@ cups_read_client_conf(
 #endif /* !__APPLE__ */
     else if (!_cups_strcasecmp(line, "User") && value)
       cups_set_user(cc, value);
+    else if (!_cups_strcasecmp(line, "UserAgentTokens") && value)
+      cups_set_uatokens(cc, value);
     else if (!_cups_strcasecmp(line, "TrustOnFirstUse") && value)
       cc->trust_first = cups_boolean_value(value);
     else if (!_cups_strcasecmp(line, "AllowAnyRoot") && value)
@@ -1252,10 +1431,10 @@ cups_read_client_conf(
     else if (!_cups_strcasecmp(line, "GSSServiceName") && value)
       cups_set_gss_service_name(cc, value);
 #endif /* HAVE_GSSAPI */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     else if (!_cups_strcasecmp(line, "SSLOptions") && value)
       cups_set_ssl_options(cc, value);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
   }
 }
 
@@ -1280,6 +1459,23 @@ cups_set_default_ipp_port(
     cg->ipp_port = CUPS_DEFAULT_IPP_PORT;
 }
 
+
+/*
+ * 'cups_set_digestoptions()' - Set the DigestOptions value.
+ */
+
+static void
+cups_set_digestoptions(
+    _cups_client_conf_t *cc,		/* I - client.conf values */
+    const char          *value)		/* I - Value */
+{
+  if (!_cups_strcasecmp(value, "DenyMD5"))
+    cc->digestoptions = _CUPS_DIGESTOPTIONS_DENYMD5;
+  else if (!_cups_strcasecmp(value, "None"))
+    cc->digestoptions = _CUPS_DIGESTOPTIONS_NONE;
+}
+
+
 /*
  * 'cups_set_encryption()' - Set the Encryption value.
  */
@@ -1332,7 +1528,7 @@ cups_set_server_name(
  * 'cups_set_ssl_options()' - Set the SSLOptions value.
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 static void
 cups_set_ssl_options(
     _cups_client_conf_t *cc,		/* I - client.conf values */
@@ -1405,7 +1601,39 @@ cups_set_ssl_options(
 
   DEBUG_printf(("4cups_set_ssl_options(cc=%p, value=\"%s\") options=%x, min_version=%d, max_version=%d", (void *)cc, value, options, min_version, max_version));
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
+
+
+/*
+ * 'cups_set_uatokens()' - Set the UserAgentTokens value.
+ */
+
+static void
+cups_set_uatokens(
+    _cups_client_conf_t *cc,		/* I - client.conf values */
+    const char          *value)		/* I - Value */
+{
+  int	i;				/* Looping var */
+  static const char * const uatokens[] =/* UserAgentTokens values */
+  {
+    "NONE",
+    "PRODUCTONLY",
+    "MAJOR",
+    "MINOR",
+    "MINIMAL",
+    "OS",
+    "FULL"
+  };
+
+  for (i = 0; i < (int)(sizeof(uatokens) / sizeof(uatokens[0])); i ++)
+  {
+    if (!_cups_strcasecmp(value, uatokens[i]))
+    {
+      cc->uatokens = (_cups_uatokens_t)i;
+      return;
+    }
+  }
+}
 
 
 /*
