@@ -537,7 +537,7 @@ ppdcSource::get_color_profile(
   for (i = 0; i < 9; i ++)
     m[i] = get_float(fp);
 
-  return (new ppdcProfile(resolution, media_type, g, d, m));
+  return (new ppdcProfile(resolution, media_type, d, g, m));
 }
 
 
@@ -1743,15 +1743,17 @@ ppdcSource::get_resolution(ppdcFile *fp)// I - File to read
 
   switch (sscanf(name, "%dx%d", &xdpi, &ydpi))
   {
-    case 0 :
-        _cupsLangPrintf(stderr,
-	                _("ppdc: Bad resolution name \"%s\" on line %d of "
-			  "%s."), name, fp->line, fp->filename);
-        break;
     case 1 :
         ydpi = xdpi;
-	break;
-  }
+        break;
+    case 2 :
+        break;
+    default :
+        _cupsLangPrintf(stderr,
+                  _("ppdc: Bad resolution name \"%s\" on line %d of "
+        "%s."), name, fp->line, fp->filename);
+        break;
+}
 
   // Create the necessary PS commands...
   snprintf(command, sizeof(command),
@@ -1878,7 +1880,7 @@ ppdcSource::get_simple_profile(ppdcFile *fp)
   }
 
   // Return the new profile...
-  return (new ppdcProfile(resolution, media_type, g, kd, m));
+  return (new ppdcProfile(resolution, media_type, kd, g, m));
 }
 
 
@@ -2790,7 +2792,7 @@ ppdcSource::scan_file(ppdcFile   *fp,	// I - File to read
       if (have_cutter <= 0 || cond_state)
         continue;
 
-      if ((o = d->find_option("CutMedia")) == NULL)
+      if (!d->find_option("CutMedia"))
       {
         o = new ppdcOption(PPDC_BOOLEAN, "CutMedia", "Cut Media", PPDC_SECTION_ANY, 10.0f);
 
@@ -2803,9 +2805,8 @@ ppdcSource::scan_file(ppdcFile   *fp,	// I - File to read
 
 	c = new ppdcChoice("True", NULL, "<</CutMedia 4>>setpagedevice");
 	o->add_choice(c);
+        o = NULL;
       }
-
-      o = NULL;
     }
     else if (!_cups_strcasecmp(temp, "Darkness"))
     {
