@@ -1,14 +1,12 @@
 /*
  * Environment management routines for the CUPS scheduler.
  *
- * Copyright 2007-2016 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products, all rights reserved.
+ * Copyright © 2021 by OpenPrinting.
+ * Copyright © 2007-2016 by Apple Inc.
+ * Copyright © 1997-2006 by Easy Software Products, all rights reserved.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * file is missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -118,11 +116,13 @@ cupsdSetEnv(const char *name,		/* I - Name of variable */
     return;
 
  /*
-  * Do not allow dynamic linker variables when running as root...
+  * Do not allow dynamic linker variables when running as root outside a Snap...
   */
 
+#if !CUPS_SNAP
   if (!RunUser && (!strncmp(name, "DYLD_", 5) || !strncmp(name, "LD_", 3)))
     return;
+#endif /* !CUPS_SNAP */
 
  /*
   * See if this variable has already been defined...
@@ -200,11 +200,11 @@ cupsdUpdateEnv(void)
   set_if_undefined("CUPS_CACHEDIR", CacheDir);
   set_if_undefined("CUPS_DATADIR", DataDir);
   set_if_undefined("CUPS_DOCROOT", DocumentRoot);
-  set_if_undefined("CUPS_FONTPATH", FontPath);
   set_if_undefined("CUPS_REQUESTROOT", RequestRoot);
   set_if_undefined("CUPS_SERVERBIN", ServerBin);
   set_if_undefined("CUPS_SERVERROOT", ServerRoot);
   set_if_undefined("CUPS_STATEDIR", StateDir);
+  set_if_undefined("DYLD_INSERT_LIBRARIES", NULL);
   set_if_undefined("DYLD_LIBRARY_PATH", NULL);
   set_if_undefined("HOME", TempDir);
   set_if_undefined("LD_ASSUME_KERNEL", NULL);
@@ -212,8 +212,16 @@ cupsdUpdateEnv(void)
   set_if_undefined("LD_PRELOAD", NULL);
   set_if_undefined("NLSPATH", NULL);
   if (find_env("PATH") < 0)
-    cupsdSetEnvf("PATH", "%s/filter:" CUPS_BINDIR ":" CUPS_SBINDIR
-			 ":/bin:/usr/bin", ServerBin);
+  {
+#if CUPS_SNAP
+    const char	*path;			// PATH environment variable
+
+    if ((path = getenv("PATH")) != NULL)
+      cupsdSetEnvf("PATH", "%s/filter:%s", ServerBin, path);
+    else
+#endif /* CUPS_SNAP */
+    cupsdSetEnvf("PATH", "%s/filter:" CUPS_BINDIR ":" CUPS_SBINDIR ":/bin:/usr/bin", ServerBin);
+  }
   set_if_undefined("SERVER_ADMIN", ServerAdmin);
   set_if_undefined("SHLIB_PATH", NULL);
   set_if_undefined("SOFTWARE", CUPS_MINIMAL);
