@@ -1,22 +1,18 @@
 /*
  * "lpadmin" command for CUPS.
  *
- * Copyright © 2007-2019 by Apple Inc.
+ * Copyright © 2021-2022 by OpenPrinting.
+ * Copyright © 2007-2021 by Apple Inc.
  * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
  * Include necessary headers...
  */
 
-#define _CUPS_NO_DEPRECATED
-#define _PPD_DEPRECATED
 #include <cups/cups-private.h>
 #include <cups/ppd-private.h>
 
@@ -33,12 +29,12 @@ static int		delete_printer_from_class(http_t *http, char *printer,
 static int		delete_printer_option(http_t *http, char *printer,
 			                      char *option);
 static int		enable_printer(http_t *http, char *printer);
-static char		*get_printer_ppd(const char *uri, char *buffer, size_t bufsize, int *num_options, cups_option_t **options);
 static cups_ptype_t	get_printer_type(http_t *http, char *printer, char *uri,
 			                 size_t urisize);
 static int		set_printer_options(http_t *http, char *printer,
 			                    int num_options, cups_option_t *options,
 					    char *file, int enable);
+static void		usage(void) _CUPS_NORETURN;
 static int		validate_name(const char *name);
 
 
@@ -75,7 +71,9 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   for (i = 1; i < argc; i ++)
   {
-    if (argv[i][0] == '-')
+    if (!strcmp(argv[i], "--help"))
+      usage();
+    else if (argv[i][0] == '-')
     {
       for (opt = argv[i] + 1; *opt; opt ++)
       {
@@ -113,7 +111,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected class name after \"-c\" option."));
-		  return (1);
+		  usage();
 		}
 
 		pclass = argv[i];
@@ -155,7 +153,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected printer name after \"-d\" option."));
-		  return (1);
+		  usage();
 		}
 
 		printer = argv[i];
@@ -192,7 +190,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected hostname after \"-h\" option."));
-		  return (1);
+		  usage();
 		}
 
 		cupsSetServer(argv[i]);
@@ -213,7 +211,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPrintf(stderr, _("lpadmin: Expected PPD after \"-%c\" option."), argv[i - 1][1]);
-		  return (1);
+		  usage();
 		}
 
 		file = argv[i];
@@ -242,14 +240,14 @@ main(int  argc,				/* I - Number of command-line arguments */
 	  case 'E' : /* Enable the printer/enable encryption */
 	      if (printer == NULL)
 	      {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 		cupsSetEncryption(HTTP_ENCRYPTION_REQUIRED);
 
 		if (http)
 		  httpEncryption(http, HTTP_ENCRYPTION_REQUIRED);
 #else
 		_cupsLangPrintf(stderr, _("%s: Sorry, no encryption support."), argv[0]);
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 		break;
 	      }
 
@@ -282,7 +280,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected model after \"-m\" option."));
-		  return (1);
+		  usage();
 		}
 
 		num_options = cupsAddOption("ppd-name", argv[i], num_options, &options);
@@ -302,7 +300,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected name=value after \"-o\" option."));
-		  return (1);
+		  usage();
 		}
 
 		num_options = cupsParseOptions(argv[i], num_options, &options);
@@ -322,7 +320,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected printer after \"-p\" option."));
-		  return (1);
+		  usage();
 		}
 
 		printer = argv[i];
@@ -369,7 +367,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected class after \"-r\" option."));
-		  return (1);
+		  usage();
 		}
 
 		pclass = argv[i];
@@ -417,7 +415,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected name after \"-R\" option."));
-		  return (1);
+		  usage();
 		}
 
 		val = argv[i];
@@ -439,7 +437,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPrintf(stderr, _("%s: Error - expected username after \"-U\" option."), argv[0]);
-		  return (1);
+		  usage();
 		}
 
 		cupsSetUser(argv[i]);
@@ -459,7 +457,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected allow/deny:userlist after \"-u\" option."));
-		  return (1);
+		  usage();
 		}
 
 		val = argv[i];
@@ -489,7 +487,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected device URI after \"-v\" option."));
-		  return (1);
+		  usage();
 		}
 
 		num_options = cupsAddOption("device-uri", argv[i], num_options, &options);
@@ -522,7 +520,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected printer or class after \"-x\" option."));
-		  return (1);
+		  usage();
 		}
 
 		printer = argv[i];
@@ -553,7 +551,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected description after \"-D\" option."));
-		  return (1);
+		  usage();
 		}
 
 		num_options = cupsAddOption("printer-info", argv[i], num_options, &options);
@@ -566,7 +564,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 		_cupsLangPuts(stderr, _("lpadmin: Expected file type(s) after \"-I\" option."));
-		return (1);
+		usage();
 	      }
 
 	      _cupsLangPuts(stderr, _("lpadmin: Warning - content type list ignored."));
@@ -585,7 +583,7 @@ main(int  argc,				/* I - Number of command-line arguments */
 		if (i >= argc)
 		{
 		  _cupsLangPuts(stderr, _("lpadmin: Expected location after \"-L\" option."));
-		  return (1);
+		  usage();
 		}
 
 		num_options = cupsAddOption("printer-location", argv[i], num_options, &options);
@@ -594,14 +592,14 @@ main(int  argc,				/* I - Number of command-line arguments */
 
 	  default :
 	      _cupsLangPrintf(stderr, _("lpadmin: Unknown option \"%c\"."), *opt);
-	      return (1);
+	      usage();
 	}
       }
     }
     else
     {
       _cupsLangPrintf(stderr, _("lpadmin: Unknown argument \"%s\"."), argv[i]);
-      return (1);
+      usage();
     }
   }
 
@@ -614,17 +612,22 @@ main(int  argc,				/* I - Number of command-line arguments */
 
   if (ppd_name && !strcmp(ppd_name, "raw"))
   {
+#ifdef __APPLE__
+    _cupsLangPuts(stderr, _("lpadmin: Raw queues are no longer supported on macOS."));
+#else
     _cupsLangPuts(stderr, _("lpadmin: Raw queues are deprecated and will stop working in a future version of CUPS."));
+#endif /* __APPLE__ */
 
     if (device_uri && (!strncmp(device_uri, "ipp://", 6) || !strncmp(device_uri, "ipps://", 7)) && strstr(device_uri, "/printers/"))
       _cupsLangPuts(stderr, _("lpadmin: Use the 'everywhere' model for shared printers."));
-  }
-  else if (ppd_name && !strcmp(ppd_name, "everywhere") && device_uri)
-  {
-    if ((file = get_printer_ppd(device_uri, evefile, sizeof(evefile), &num_options, &options)) == NULL)
-      return (1);
 
-    num_options = cupsRemoveOption("ppd-name", num_options, &options);
+#ifdef __APPLE__
+    return (1);
+#endif /* __APPLE__ */
+  }
+  else if ((ppd_name && strcmp(ppd_name, "everywhere") && strncmp(ppd_name, "driverless:", 11)) || file)
+  {
+    _cupsLangPuts(stderr, _("lpadmin: Printer drivers are deprecated and will stop working in a future version of CUPS."));
   }
 
   if (num_options || file)
@@ -659,20 +662,7 @@ main(int  argc,				/* I - Number of command-line arguments */
     unlink(evefile);
 
   if (printer == NULL)
-  {
-    _cupsLangPuts(stdout,
-	          _("Usage:\n"
-		    "\n"
-		    "    lpadmin [-h server] -d destination\n"
-		    "    lpadmin [-h server] -x destination\n"
-		    "    lpadmin [-h server] -p printer [-c add-class] "
-		    "[-i interface] [-m model]\n"
-		    "                       [-r remove-class] [-v device] "
-		    "[-D description]\n"
-		    "                       [-P ppd-file] [-o name=value]\n"
-		    "                       [-u allow:user,user] "
-		    "[-u deny:user,user]"));
-  }
+    usage();
 
   if (http)
     httpClose(http);
@@ -698,9 +688,6 @@ add_printer_to_class(http_t *http,	/* I - Server connection */
   char		uri[HTTP_MAX_URI];	/* URI for printer/class */
 
 
-  DEBUG_printf(("add_printer_to_class(%p, \"%s\", \"%s\")\n", http,
-                printer, pclass));
-
  /*
   * Build an IPP_OP_GET_PRINTER_ATTRIBUTES request, which requires the following
   * attributes:
@@ -819,8 +806,6 @@ default_printer(http_t *http,		/* I - Server connection */
   char		uri[HTTP_MAX_URI];	/* URI for printer/class */
 
 
-  DEBUG_printf(("default_printer(%p, \"%s\")\n", http, printer));
-
  /*
   * Build a CUPS-Set-Default request, which requires the following
   * attributes:
@@ -870,8 +855,6 @@ delete_printer(http_t *http,		/* I - Server connection */
   char		uri[HTTP_MAX_URI];	/* URI for printer/class */
 
 
-  DEBUG_printf(("delete_printer(%p, \"%s\")\n", http, printer));
-
  /*
   * Build a CUPS-Delete-Printer request, which requires the following
   * attributes:
@@ -926,9 +909,6 @@ delete_printer_from_class(
   char		uri[HTTP_MAX_URI];	/* URI for printer/class */
 
 
-  DEBUG_printf(("delete_printer_from_class(%p, \"%s\", \"%s\")\n", http,
-                printer, pclass));
-
  /*
   * Build an IPP_OP_GET_PRINTER_ATTRIBUTES request, which requires the following
   * attributes:
@@ -1126,8 +1106,6 @@ enable_printer(http_t *http,		/* I - Server connection */
   char		uri[HTTP_MAX_URI];	/* URI for printer/class */
 
 
-  DEBUG_printf(("enable_printer(%p, \"%s\")\n", http, printer));
-
  /*
   * Send IPP_OP_ENABLE_PRINTER and IPP_OP_RESUME_PRINTER requests, which
   * require the following attributes:
@@ -1171,105 +1149,6 @@ enable_printer(http_t *http,		/* I - Server connection */
 }
 
 
-/*
- * 'get_printer_ppd()' - Get an IPP Everywhere PPD file for the given URI.
- */
-
-static char *				/* O  - Filename or NULL */
-get_printer_ppd(
-    const char    *uri,			/* I  - Printer URI */
-    char          *buffer,		/* I  - Filename buffer */
-    size_t        bufsize,		/* I  - Size of filename buffer */
-    int           *num_options,		/* IO - Number of options */
-    cups_option_t **options)		/* IO - Options */
-{
-  http_t	*http;			/* Connection to printer */
-  ipp_t		*request,		/* Get-Printer-Attributes request */
-		*response;		/* Get-Printer-Attributes response */
-  ipp_attribute_t *attr;		/* Attribute from response */
-  char		resolved[1024],		/* Resolved URI */
-		scheme[32],		/* URI scheme */
-		userpass[256],		/* Username:password */
-		host[256],		/* Hostname */
-		resource[256];		/* Resource path */
-  int		port;			/* Port number */
-  static const char * const pattrs[] =	/* Attributes to use */
-  {
-    "all",
-    "media-col-database"
-  };
-
-
- /*
-  * Connect to the printer...
-  */
-
-  if (strstr(uri, "._tcp"))
-  {
-   /*
-    * Resolve URI...
-    */
-
-    if (!_httpResolveURI(uri, resolved, sizeof(resolved), _HTTP_RESOLVE_DEFAULT, NULL, NULL))
-    {
-      _cupsLangPrintf(stderr, _("%s: Unable to resolve \"%s\"."), "lpadmin", uri);
-      return (NULL);
-    }
-
-    uri = resolved;
-  }
-
-  if (httpSeparateURI(HTTP_URI_CODING_ALL, uri, scheme, sizeof(scheme), userpass, sizeof(userpass), host, sizeof(host), &port, resource, sizeof(resource)) < HTTP_URI_STATUS_OK)
-  {
-    _cupsLangPrintf(stderr, _("%s: Bad printer URI \"%s\"."), "lpadmin", uri);
-    return (NULL);
-  }
-
-  http = httpConnect2(host, port, NULL, AF_UNSPEC, !strcmp(scheme, "ipps") ? HTTP_ENCRYPTION_ALWAYS : HTTP_ENCRYPTION_IF_REQUESTED, 1, 30000, NULL);
-  if (!http)
-  {
-    _cupsLangPrintf(stderr, _("%s: Unable to connect to \"%s:%d\": %s"), "lpadmin", host, port, cupsLastErrorString());
-    return (NULL);
-  }
-
- /*
-  * Send a Get-Printer-Attributes request...
-  */
-
-  request = ippNewRequest(IPP_OP_GET_PRINTER_ATTRIBUTES);
-  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI, "printer-uri", NULL, uri);
-  ippAddStrings(request, IPP_TAG_OPERATION, IPP_TAG_KEYWORD, "requested-attributes", sizeof(pattrs) / sizeof(pattrs[0]), NULL, pattrs);
-  response = cupsDoRequest(http, request, resource);
-
-  if (cupsLastError() >= IPP_STATUS_REDIRECTION_OTHER_SITE)
-  {
-    _cupsLangPrintf(stderr, _("%s: Unable to query printer: %s"), "lpadmin", cupsLastErrorString());
-    buffer[0] = '\0';
-  }
-  else if (_ppdCreateFromIPP(buffer, bufsize, response))
-  {
-    if (!cupsGetOption("printer-geo-location", *num_options, *options) && (attr = ippFindAttribute(response, "printer-geo-location", IPP_TAG_URI)) != NULL)
-      *num_options = cupsAddOption("printer-geo-location", ippGetString(attr, 0, NULL), *num_options, options);
-
-    if (!cupsGetOption("printer-info", *num_options, *options) && (attr = ippFindAttribute(response, "printer-info", IPP_TAG_TEXT)) != NULL)
-      *num_options = cupsAddOption("printer-info", ippGetString(attr, 0, NULL), *num_options, options);
-
-    if (!cupsGetOption("printer-location", *num_options, *options) && (attr = ippFindAttribute(response, "printer-location", IPP_TAG_TEXT)) != NULL)
-      *num_options = cupsAddOption("printer-location", ippGetString(attr, 0, NULL), *num_options, options);
-  }
-  else
-    _cupsLangPrintf(stderr, _("%s: Unable to create PPD file: %s"), "lpadmin", strerror(errno));
-
-  ippDelete(response);
-  httpClose(http);
-
-  if (buffer[0])
-    return (buffer);
-  else
-    return (NULL);
-}
-
-
 /*
  * 'get_printer_type()' - Determine the printer type and URI.
  */
@@ -1363,10 +1242,6 @@ set_printer_options(
 		copied_options = 0;	/* Copied options? */
 
 
-  DEBUG_printf(("set_printer_options(http=%p, printer=\"%s\", num_options=%d, "
-                "options=%p, file=\"%s\")\n", http, printer, num_options,
-		options, file));
-
  /*
   * Build a CUPS-Add-Modify-Printer or CUPS-Add-Modify-Class request,
   * which requires the following attributes:
@@ -1392,7 +1267,7 @@ set_printer_options(
 
   if (file)
     ppdfile = file;
-  else if ((ppdname = cupsGetOption("ppd-name", num_options, options)) != NULL && strcmp(ppdname, "raw") && num_options > 1)
+  else if ((ppdname = cupsGetOption("ppd-name", num_options, options)) != NULL && strcmp(ppdname, "everywhere") && strcmp(ppdname, "raw") && num_options > 1)
   {
     if ((ppdfile = cupsGetServerPPD(http, ppdname)) != NULL)
     {
@@ -1450,7 +1325,7 @@ set_printer_options(
 					/* Status code */
 
       _cupsLangPrintf(stderr, _("lpadmin: Unable to open PPD \"%s\": %s on line %d."), ppdfile, ppdErrorString(status), linenum);
-      return (1);
+      goto error;
     }
 
     ppdMarkDefaults(ppd);
@@ -1459,27 +1334,15 @@ set_printer_options(
     if ((out = cupsTempFile2(tempfile, sizeof(tempfile))) == NULL)
     {
       _cupsLangPrintError(NULL, _("lpadmin: Unable to create temporary file"));
-      ippDelete(request);
-      if (ppdfile != file)
-        unlink(ppdfile);
-      if (copied_options)
-        cupsFreeOptions(num_options, options);
-      return (1);
+      goto error;
     }
 
     if ((in = cupsFileOpen(ppdfile, "r")) == NULL)
     {
-      _cupsLangPrintf(stderr,
-                      _("lpadmin: Unable to open PPD file \"%s\" - %s"),
-        	      ppdfile, strerror(errno));
-      ippDelete(request);
-      if (ppdfile != file)
-	unlink(ppdfile);
-      if (copied_options)
-        cupsFreeOptions(num_options, options);
+      _cupsLangPrintf(stderr, _("lpadmin: Unable to open PPD \"%s\": %s"), ppdfile, strerror(errno));
       cupsFileClose(out);
       unlink(tempfile);
-      return (1);
+      goto error;
     }
 
     while (cupsFileGets(in, line, sizeof(line)))
@@ -1623,6 +1486,74 @@ set_printer_options(
   }
   else
     return (0);
+
+ /*
+  * Error handling...
+  */
+
+  error:
+
+  ippDelete(request);
+
+  if (ppdfile != file)
+    unlink(ppdfile);
+
+  if (copied_options)
+    cupsFreeOptions(num_options, options);
+
+  return (1);
+}
+
+
+/*
+ * 'usage()' - Show program usage and exit.
+ */
+
+static void
+usage(void)
+{
+  _cupsLangPuts(stdout, _("Usage: lpadmin [options] -d destination\n"
+                          "       lpadmin [options] -p destination\n"
+                          "       lpadmin [options] -p destination -c class\n"
+                          "       lpadmin [options] -p destination -r class\n"
+                          "       lpadmin [options] -x destination"));
+  _cupsLangPuts(stdout, _("Options:"));
+  _cupsLangPuts(stdout, _("-c class                Add the named destination to a class"));
+  _cupsLangPuts(stdout, _("-d destination          Set the named destination as the server default"));
+  _cupsLangPuts(stdout, _("-D description          Specify the textual description of the printer"));
+  _cupsLangPuts(stdout, _("-E                      Encrypt the connection to the server"));
+  _cupsLangPuts(stdout, _("-E                      Enable and accept jobs on the printer (after -p)"));
+  _cupsLangPuts(stdout, _("-h server[:port]        Connect to the named server and port"));
+  _cupsLangPuts(stdout, _("-i ppd-file             Specify a PPD file for the printer"));
+  _cupsLangPuts(stdout, _("-L location             Specify the textual location of the printer"));
+  _cupsLangPuts(stdout, _("-m model                Specify a standard model/PPD file for the printer"));
+  _cupsLangPuts(stdout, _("-m everywhere           Specify the printer is compatible with IPP Everywhere"));
+  _cupsLangPuts(stdout, _("-o name-default=value   Specify the default value for the named option"));
+  _cupsLangPuts(stdout, _("-o Name=Value           Specify the default value for the named PPD option "));
+  _cupsLangPuts(stdout, _("-o cupsIPPSupplies=false\n"
+                          "                        Disable supply level reporting via IPP"));
+  _cupsLangPuts(stdout, _("-o cupsSNMPSupplies=false\n"
+                          "                        Disable supply level reporting via SNMP"));
+  _cupsLangPuts(stdout, _("-o job-k-limit=N        Specify the kilobyte limit for per-user quotas"));
+  _cupsLangPuts(stdout, _("-o job-page-limit=N     Specify the page limit for per-user quotas"));
+  _cupsLangPuts(stdout, _("-o job-quota-period=N   Specify the per-user quota period in seconds"));
+  _cupsLangPuts(stdout, _("-o printer-error-policy=name\n"
+                          "                        Specify the printer error policy"));
+  _cupsLangPuts(stdout, _("-o printer-is-shared=true\n"
+                          "                        Share the printer"));
+  _cupsLangPuts(stdout, _("-o printer-op-policy=name\n"
+                          "                        Specify the printer operation policy"));
+  _cupsLangPuts(stdout, _("-p destination          Specify/add the named destination"));
+  _cupsLangPuts(stdout, _("-r class                Remove the named destination from a class"));
+  _cupsLangPuts(stdout, _("-R name-default         Remove the default value for the named option"));
+  _cupsLangPuts(stdout, _("-u allow:all            Allow all users to print"));
+  _cupsLangPuts(stdout, _("-u allow:list           Allow the list of users or groups (@name) to print"));
+  _cupsLangPuts(stdout, _("-u deny:list            Prevent the list of users or groups (@name) to print"));
+  _cupsLangPuts(stdout, _("-U username             Specify the username to use for authentication"));
+  _cupsLangPuts(stdout, _("-v device-uri           Specify the device URI for the printer"));
+  _cupsLangPuts(stdout, _("-x destination          Remove the named destination"));
+
+  exit(1);
 }
 
 
