@@ -1,8 +1,9 @@
 /*
  * Configuration file for CUPS and Xcode.
  *
- * Copyright 2007-2019 by Apple Inc.
- * Copyright 1997-2007 by Easy Software Products.
+ * Copyright © 2021-2022 by OpenPrinting
+ * Copyright © 2007-2019 by Apple Inc.
+ * Copyright © 1997-2007 by Easy Software Products.
  *
  * Licensed under Apache License v2.0.  See the file "LICENSE" for more information.
  */
@@ -18,8 +19,8 @@
  * Version of software...
  */
 
-#define CUPS_SVERSION "CUPS v2.3.1"
-#define CUPS_MINIMAL "CUPS/2.3.1"
+#define CUPS_SVERSION "CUPS v2.4.2"
+#define CUPS_MINIMAL "CUPS/2.4.2"
 
 
 /*
@@ -47,6 +48,7 @@
 
 #define CUPS_DEFAULT_LOG_LEVEL "warn"
 #define CUPS_DEFAULT_ACCESS_LOG_LEVEL "none"
+#define CUPS_DEFAULT_MAX_LOG_SIZE "1m"
 
 
 /*
@@ -80,11 +82,10 @@
 
 
 /*
- * Default Samba and LPD config files...
+ * Default ErrorPolicy value...
  */
 
-#define CUPS_DEFAULT_SMB_CONFIG_FILE ""
-#define CUPS_DEFAULT_LPD_CONFIG_FILE "launchd:///System/Library/LaunchDaemons/org.cups.cups-lpd.plist"
+#define CUPS_DEFAULT_ERROR_POLICY "stop-printer"
 
 
 /*
@@ -94,6 +95,13 @@
 #define CUPS_DEFAULT_MAX_COPIES 9999
 
 
+/*
+ * Default SyncOnClose value...
+ */
+
+/* #undef CUPS_DEFAULT_SYNC_ON_CLOSE */
+
+
 /*
  * Do we have domain socket support, and if so what is the default one?
  */
@@ -119,7 +127,6 @@
 #define CUPS_CACHEDIR "/private/var/spool/cups/cache"
 #define CUPS_DATADIR "/usr/share/cups"
 #define CUPS_DOCROOT "/usr/share/doc/cups"
-#define CUPS_FONTPATH "/usr/share/cups/fonts"
 #define CUPS_LOCALEDIR "/usr/share/locale"
 #define CUPS_LOGDIR "/private/var/log/cups"
 #define CUPS_REQUESTS "/private/var/spool/cups"
@@ -299,10 +306,11 @@
  * Which encryption libraries do we have?
  */
 
+#define HAVE_TLS 1
 #define HAVE_CDSASSL 1
+/* #undef HAVE_OPENSSL */
 /* #undef HAVE_GNUTLS */
 /* #undef HAVE_SSPISSL */
-#define HAVE_SSL 1
 
 
 /*
@@ -349,14 +357,21 @@
 
 
 /*
- * Do we have mDNSResponder for DNS Service Discovery (aka Bonjour)?
+ * Do we have DNS Service Discovery (aka Bonjour) support?
  */
 
 #define HAVE_DNSSD 1
 
 
 /*
- * Do we have Avahi for DNS Service Discovery (aka Bonjour)?
+ * Do we have mDNSResponder for DNS-SD?
+ */
+
+#define HAVE_MDNSRESPONDER 1
+
+
+/*
+ * Do we have Avahi for DNS-SD?
  */
 
 /* #undef HAVE_AVAHI */
@@ -544,7 +559,7 @@
  */
 
 #if TARGET_OS_OSX
-#  define HAVE_GSS_ACQUIRED_CRED_EX_F 1
+#  define HAVE_GSS_ACQUIRE_CRED_EX_F 1
 #  define HAVE_GSS_C_NT_HOSTBASED_SERVICE 1
 #  define HAVE_GSS_GSSAPI_H 1
 /* #undef HAVE_GSS_GSSAPI_SPI_H */
